#include <string>
#include <iostream>

#include "buffer.h"

#include "debug.h"

Xr_BufferManager* Xr_BufferManager::B_instance = 0;

Xr_BufferManager::Xr_BufferManager()
{
}

Xr_BufferManager::~Xr_BufferManager()
{
	unloadAll();
}

Xr_BufferManager* Xr_BufferManager::getInstance()
{
	if(B_instance)
	{
		return B_instance;
	}
	else
	{
		B_instance = new Xr_BufferManager;
		return B_instance;
	}
}

bool Xr_BufferManager::instanced()
{
	if(B_instance)
	{
		return true;
	}
	else
	{
		return false;
	}
}

void Xr_BufferManager::quitInstance()
{
	if(B_instance)
	{
		delete B_instance;
	}
}

bool Xr_BufferManager::load(Xr_Mesh *mesh)
{
	int i;
	
	if(mesh)
	{
		for(i = 0; i < B_meshList.size(); i++)
		{
			if(B_meshList[i].mesh == mesh)
			{
				return true;
			}
		}
		
		Xr_Log("Xr_BufferManager : Chargement des données géométriques du mesh < " + std::string(mesh->getName()) + " >");
		if(mesh->getFaceNumber() == 0)
		{
			Xr_Log("Xr_BufferManager : Le mesh ne contient aucune face");
			return false;
		}
		
		if(mesh->getVertex())
		{
			B_meshList.push_back(meshDesc());
			B_meshList.back().mesh = mesh;
			B_meshList.back().vertexSize = ((mesh->getFaceNumber() * 3) * 3) * sizeof(float);
			B_meshList.back().vertexOffset = 0;
		}
		else
		{
			Xr_Log("Xr_BufferManager : Le mesh ne contient aucun vertice");
			B_meshList.erase(B_meshList.end());
			return false;
		}
		
		if(mesh->getTexCoord())
		{
			B_meshList.back().texCoordSize = ((mesh->getFaceNumber() * 3) * 2) * sizeof(float);
			B_meshList.back().texCoordOffset = B_meshList.back().vertexSize;
		}
		
		if(mesh->getNormal())
		{
			B_meshList.back().normalSize = ((mesh->getFaceNumber() * 3) * 3) * sizeof(float);
			B_meshList.back().normalOffset = B_meshList.back().vertexSize + B_meshList.back().texCoordSize;
		}
		
		glGenBuffers(1, &(B_meshList.back().vbo));
		glBindBuffer(GL_ARRAY_BUFFER, B_meshList.back().vbo);
		glBufferData(GL_ARRAY_BUFFER, (GLsizei) B_meshList.back().vertexSize + B_meshList.back().texCoordSize + B_meshList.back().normalSize, 0, GL_STREAM_DRAW);
		glBufferSubData(GL_ARRAY_BUFFER, (GLint) B_meshList.back().vertexOffset, (GLsizei) B_meshList.back().vertexSize, mesh->getVertex());
		glBufferSubData(GL_ARRAY_BUFFER, (GLint) B_meshList.back().texCoordOffset, (GLsizei) B_meshList.back().texCoordSize, mesh->getTexCoord());
		glBufferSubData(GL_ARRAY_BUFFER, (GLint) B_meshList.back().normalOffset, (GLsizei) B_meshList.back().normalSize, mesh->getNormal());
		
		return true;
	}
	else
	{
		Xr_Log("Xr_BufferManager : Le mesh est invalide");
	}
}

void Xr_BufferManager::unload(Xr_Mesh *mesh)
{
	int i;
	
	for(i = 0; i < B_meshList.size(); i++)
	{
		if(B_meshList[i].mesh == mesh)
		{
			Xr_Log("Xr_BufferManager : Suppression des données géométriques du mesh < " + std::string(mesh->getName()) + " >");
			glDeleteBuffers(1, &(B_meshList[i].vbo));
			B_meshList.erase(B_meshList.begin() + i);
			return;
		}
	}
}


void Xr_BufferManager::unloadAll()
{
	int i;
	
	for(i = 0; i < B_meshList.size(); i++)
	{
		Xr_Log("Xr_BufferManager : Suppression des données géométriques du mesh < " + std::string(B_meshList[i].mesh->getName()) + " >");
		glDeleteBuffers(1, &(B_meshList[i].vbo));
	}
	B_meshList.clear();
}

void Xr_BufferManager::fetchVertex(Xr_Mesh *mesh)
{
	int i;
	for(i = 0; i < B_meshList.size(); i++)
	{
		if(B_meshList[i].mesh == mesh)
		{
			glBindBuffer(GL_ARRAY_BUFFER, B_meshList[i].vbo);
			glVertexPointer(3, GL_FLOAT, 0, (float*) B_meshList[i].vertexOffset);
			glEnableClientState(GL_VERTEX_ARRAY);
			return;
		}
	}
}

void Xr_BufferManager::fetchTexCoord(Xr_Mesh *mesh)
{
	int i;
	for(i = 0; i < B_meshList.size(); i++)
	{
		if(B_meshList[i].mesh == mesh)
		{
			glBindBuffer(GL_ARRAY_BUFFER, B_meshList[i].vbo);
			glTexCoordPointer(2, GL_FLOAT, 0,(float*) B_meshList[i].texCoordOffset);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			return;
		}
	}
}

void Xr_BufferManager::fetchNormal(Xr_Mesh *mesh)
{
	int i;
	for(i = 0; i < B_meshList.size(); i++)
	{
		if(B_meshList[i].mesh == mesh)
		{
			glBindBuffer(GL_ARRAY_BUFFER, B_meshList[i].vbo);
			glNormalPointer(GL_FLOAT, 0,(float*) B_meshList[i].normalOffset);
			glEnableClientState(GL_NORMAL_ARRAY);
			return;
		}
	}
}

void Xr_BufferManager::fetchNull()
{
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
}
