#include <string>

#include "program.h"

#include "debug.h"

Xr_Program::Xr_Program() : P_id(0), P_shader1(0), P_shader2(0), P_isLink(false)
{
}

Xr_Program::~Xr_Program()
{
	glDeleteProgram(P_id);
}

void Xr_Program::attachShader(Xr_Shader *shader)
{
	if(shader)
	{
		if(shader->isCompile())
		{
			if(P_shader1 == 0)
			{
				P_shader1 = shader;
			}
			else if(P_shader2 == 0)
			{
				P_shader2 = shader;
			}
			else
			{
				Xr_Log("Xr_Program : Des shaders sont déjà attaché au program");
				return;
			}
		}
		else
		{
			Xr_Log("Xr_Program : Le shader n'est pas compilé");
			return;
		}
	}
	else
	{
		Xr_Log("Xr_Program : Le shader est invalide");
	}
}

void Xr_Program::detachShader(Xr_Shader *shader)
{
	if(P_shader1 == shader)
	{
		P_shader1 = 0;
	}
	else if(P_shader2 == shader)
	{
		P_shader2 = 0;
	}
}

void Xr_Program::link()
{
	GLint status = GL_TRUE;
	
	if(P_shader1 && P_shader2)
	{
		P_id = glCreateProgram();
		if(P_id == 0)
		{
			Xr_Log("Xr_Program : Impossible de créer un program de shaders");
			P_isLink = false;
			return;
		}
	
		glAttachShader(P_id, P_shader1->getShaderId());
		glAttachShader(P_id, P_shader2->getShaderId());
		glLinkProgram(P_id);
	
		glGetProgramiv(P_id, GL_LINK_STATUS, &status);
	
		if(status != GL_TRUE)
		{
			GLint logSize;
			glGetProgramiv(P_id, GL_INFO_LOG_LENGTH, &logSize);
		
			char *log = new char[logSize + 1];
		
			glGetProgramInfoLog(P_id, logSize, &logSize, log);
			log[logSize] = 0;
		
			Xr_Log("Xr_Program : Le linkage des shaders < " + std::string(P_shader1->getName()) + " > et < " + std::string(P_shader2->getName()) + " > a échoué\n[OPENGL_GLSL] : " + std::string(log));
			delete[] log;
			P_isLink = false;
			return;
		}
		Xr_Log("Xr_Program : Les shaders < " + std::string(P_shader1->getName()) + " > et < " + std::string(P_shader2->getName()) + " ont été linkés");
		P_isLink = true;
	}
	else
	{
		Xr_Log("Xr_Program : Le program ne contient pas deux shaders");
	}
}

bool Xr_Program::isLink() const
{
	return P_isLink;
}

GLuint Xr_Program::getProgramId() const
{
	return P_id;
}
