#include <string>

#include "ressource.h"

#include "mesh.h"
#include "texture.h"
#include "shader.h"
#include "debug.h"

/* class Xr_Ressource */

Xr_Ressource::Xr_Ressource() : R_name(0), R_isLoaded(false)
{
}

Xr_Ressource::Xr_Ressource(char *name, ressourceType type) : R_name(name), R_isLoaded(false), R_type(type)
{
	
}

Xr_Ressource::Xr_Ressource(ressourceType type) : R_name(0), R_isLoaded(false), R_type(type)
{
}

Xr_Ressource::~Xr_Ressource()
{
}

char* const Xr_Ressource::getName() const
{
	return (char* const) R_name;
}

bool Xr_Ressource::isLoaded() const
{
	return R_isLoaded;
}

ressourceType Xr_Ressource::getType() const
{
	return R_type;
}

/* class Xr_RessourceManager */

Xr_RessourceManager* Xr_RessourceManager::R_instance = 0;

Xr_RessourceManager::Xr_RessourceManager()
{
}

Xr_RessourceManager::~Xr_RessourceManager()
{
	unloadAll();
}

bool Xr_RessourceManager::instanced()
{
	if(R_instance)
	{
		return true;
	}
	else
	{
		return false;
	}
}

Xr_RessourceManager* Xr_RessourceManager::getInstance()
{
	if(R_instance)
	{
		return R_instance;
	}
	else
	{
		R_instance = new Xr_RessourceManager;
		return R_instance;
	}
}

void Xr_RessourceManager::quitInstance()
{
	if(R_instance)
	{
		delete R_instance;
	}
}

Xr_Ressource* Xr_RessourceManager::get(ressourceType type, char *name)
{
	int i;
	std::string Rname((const char*) name);
	
	for(i = 0; i < R_ressourcesList.size(); i++)
	{
		if(std::string((const char*) R_ressourcesList[i]->getName()) == Rname && type == R_ressourcesList[i]->getType())
		{
			return R_ressourcesList[i];
		}
	}
	
	Xr_Ressource *ressource = 0;

	if(type == xr_mesh_type)
	{
		ressource = new Xr_Mesh(name);
		
		if(ressource->isLoaded())
		{
			Xr_Log("Xr_RessourceManager : Le mesh < " + Rname + " > est chargé");
			R_ressourcesList.push_back(ressource);
			return ressource;
		}
		else
		{
			Xr_Log("Xr_RessourceManager : Le chargement du mesh < " + Rname + " > a échoué");
			delete ressource;
			return 0;
		}	
	}
	else if(type == xr_texture_type)
	{
		ressource = new Xr_Texture(name);
		
		if(ressource->isLoaded())
		{
			Xr_Log("Xr_RessourceManager : La texture < " + Rname + " > est chargée");
			R_ressourcesList.push_back(ressource);
			return ressource;
		}
		else
		{
			Xr_Log("Xr_RessourceManager : Le chargement de la texture < " + Rname + " > a échoué");
			delete ressource;
			return 0;
		}	
	}
	else if(type == xr_shader_type)
	{
		ressource = new Xr_Shader(name);
		
		if(ressource->isLoaded() && ((Xr_Shader*) ressource)->isCompile())
		{
			Xr_Log("Xr_RessourceManager : Le shader < " + Rname + " > est chargé et compilé");
			R_ressourcesList.push_back(ressource);
			return ressource;
		}
		else
		{
			Xr_Log("Xr_RessourceManager : Le chargement du shader < " + Rname + " > a échoué");
			delete ressource;
			return 0;
		}	
	}
}

bool Xr_RessourceManager::load(ressourceType type, char *name)
{
	if(get(type, name))
	{
		return true;
	}
	else
	{
		return false;
	}
}

void Xr_RessourceManager::unload(ressourceType type, char *name)
{
	int i;
	std::string Rname((const char*) name);
	
	for(i = 0; i < R_ressourcesList.size(); i++)
	{
		if(std::string((const char*) R_ressourcesList[i]->getName()) == Rname && type == R_ressourcesList[i]->getType())
		{
			Xr_Log("Xr_RessourceManager : Suppression de < " + Rname + " >");
			delete R_ressourcesList[i];
			R_ressourcesList.erase(R_ressourcesList.begin() + i);
			return;
		}
	}
}

void Xr_RessourceManager::unloadAll()
{
	int i;
	for(i = 0; i < R_ressourcesList.size(); i++)
	{
		Xr_Log("Xr_RessourceManager : Suppression de < " + std::string((const char*) R_ressourcesList[i]->getName()) + " >");
		delete R_ressourcesList[i];
	}
	
	R_ressourcesList.clear();
}
