#include <fstream>
#include <string>

#include "shader.h"

#include "debug.h"


Xr_Shader::Xr_Shader() : Xr_Ressource(xr_shader_type), S_id(0), S_src(0), S_isCompile(false), S_srcLength(0)
{
}

Xr_Shader::Xr_Shader(char *name) : Xr_Ressource(name, xr_shader_type), S_id(0), S_src(0), S_isCompile(false), S_srcLength(0)
{
	load();
	compile();
}

Xr_Shader::~Xr_Shader()
{
	glDeleteShader(S_id);
	if(S_src)
	{
		delete[] S_src;
	}
}

void Xr_Shader::load()
{
	if(R_name)
	{
		if(std::string(R_name).rfind(".vert") != std::string::npos)
		{
			S_type = GL_VERTEX_SHADER;
		}
		else if(std::string(R_name).rfind(".frag") != std::string::npos)
		{
			S_type = GL_FRAGMENT_SHADER;
		}
		else
		{
			return;
		}
	
		std::ifstream flux;
		flux.open(R_name, std::ios::binary);
	
		if(flux)
		{
			//taille du fichier
			flux.seekg(0, std::ios::end);
			S_srcLength = flux.tellg();
			flux.seekg(0, std::ios::beg);
	
			S_src = new char[S_srcLength + 1];
	
			flux.read(S_src, S_srcLength);
			S_src[S_srcLength] = 0;
	
			flux.close();
	
			S_id = glCreateShader(S_type);
	
			if(S_id == 0)
			{
				Xr_Log("Xr_Shader : Erreur lors de la création du shader");
				return;
			}
	
			glShaderSource(S_id, 1, (const GLchar**) &S_src, NULL);
			R_isLoaded = true;
		}
	}
}

void Xr_Shader::compile()
{
	if(R_isLoaded)
	{
		GLint status = GL_TRUE;

		glCompileShader(S_id);
		glGetShaderiv(S_id, GL_COMPILE_STATUS, &status);
	
		if(status != GL_TRUE)
		{
			GLint logSize;
			glGetShaderiv(S_id, GL_INFO_LOG_LENGTH, &logSize);
		
			char *log = new char[logSize + 1];
		
			glGetShaderInfoLog(S_id, logSize, &logSize, log);
			log[logSize] = 0;
		
			Xr_Log("Xr_Shader : La compilation du shader < " + std::string(R_name) + " > a échouée\n[OPENGL_GLSL] : " + std::string(log));
			delete[] log;
			S_isCompile = false;
			return;
		}
		S_isCompile = true;
	}
}

GLuint Xr_Shader::getShaderId() const
{
	return S_id;
}

GLenum Xr_Shader::getShaderType() const
{
	return S_type;
}

int Xr_Shader::getSrcLength() const
{
	return S_srcLength;
}

bool Xr_Shader::isCompile() const
{
	return S_isCompile;
}

char* const Xr_Shader::getSrc() const
{
	return (char* const) S_src;
}
