#include <SDL/SDL_image.h>
#include <SDL/SDL.h>
#include <string>

#include "texture.h"

#include "debug.h"

Xr_Texture::Xr_Texture() : Xr_Ressource(xr_texture_type), T_id(0)
{
}

Xr_Texture::Xr_Texture(char *name) : Xr_Ressource(name, xr_texture_type), T_id(0)
{
	load();
}

Xr_Texture::~Xr_Texture()
{
	glDeleteTextures(1, &T_id);
}

void Xr_Texture::load()
{
	if(R_name)
	{
		SDL_Surface *image = NULL;
		SDL_Surface *final = NULL;
		Uint32 rmask, gmask, bmask, amask;
		
		//chargement de l'image avec SDL
		image = IMG_Load(R_name);
		if(image == NULL)
		{
			return;
		}
		
		//model de format pour OPENGL
		rmask = 0x000000ff;
    	gmask = 0x0000ff00;
    	bmask = 0x00ff0000;
    	amask = 0xff000000;
    	
    	SDL_PixelFormat format = *(image->format);
    	format.BitsPerPixel = 32;
    	format.BytesPerPixel = 4;
    	format.Rmask = rmask;
    	format.Gmask = gmask;
    	format.Bmask = bmask;
    	format.Amask = amask;
    	
    	//convertion vers le format définit
    	final = SDL_ConvertSurface(image,&format,SDL_SWSURFACE);
		
		glGenTextures(1, &T_id);
		glActiveTexture(GL_TEXTURE0);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, T_id);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		
		//envoie des pixels de l'image traiter à OPENGL
		glTexImage2D(GL_TEXTURE_2D, 0, 4, final->w, final->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, final->pixels);
		
		
		glBindTexture(GL_TEXTURE_2D, 0);
		SDL_FreeSurface(image);
		SDL_FreeSurface(final);

		R_isLoaded = true;
	}
}

GLuint Xr_Texture::getTextureId() const
{
	return T_id;
}
